/* SCE CONFIDENTIAL
PhyreEngine(TM) Package 3.1.0.0
* Copyright (C) 2011 Sony Computer Entertainment Inc.
* All Rights Reserved.
*/

#ifndef PHYRE_DEFAULT_SHADER_SHARED_CODE_H
#define PHYRE_DEFAULT_SHADER_SHARED_CODE_H

#include "PhyreShaderCommon.h"

// Shared code and shader implementations.

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Context Switch definitions


// The maximum number of lights this shader supports.
#define MAX_NUM_LIGHTS 3

// Context switches
bool PhyreContextSwitches 
< 
string ContextSwitchNames[] = {"NUM_LIGHTS", "LOD_BLEND", "INSTANCING_ENABLED", "SHADER_LOD_LEVEL"}; 
int MaxNumLights = MAX_NUM_LIGHTS; 
string SupportedLightTypes[] = {"DirectionalLight","PointLight","SpotLight"};
string SupportedShadowTypes[] = {"PCFShadowMap", "CascadedShadowMap"};//, "CombinedCascadedShadowMap"};
int NumSupportedShaderLODLevels = 1;
>;

#ifdef SHADER_LOD_LEVEL

#if SHADER_LOD_LEVEL > 0
	#ifdef NORMAL_MAPPING_ENABLED
		#undef NORMAL_MAPPING_ENABLED
	#endif	
#endif


#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Preprocessing of switches. 
// Setup macros for common combinations of switches
#ifdef NUM_LIGHTS
#if NUM_LIGHTS > MAX_NUM_LIGHTS
#error Maximum number of supported lights exceeded.
#endif
#endif

#ifdef NUM_LIGHTS
#if defined(LIGHTING_ENABLED) && NUM_LIGHTS > 0
#define USE_LIGHTING
#endif
#endif

#if defined(NORMAL_MAPPING_ENABLED)
#define USE_TANGENTS
#endif

#if defined(ALPHA_ENABLED) && defined(TEXTURE_ENABLED)
#define SHADOW_TEXTURE_SAMPLING_ENABLED
#define ZPREPASS_TEXTURE_SAMPLING_ENABLED
#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Global shader parameters.

// Un-tweakables
float4x4 World		: World;		
float4x4 WorldView	: WorldView;		
float4x4 WorldInverse	: WorldInverse;
float4x4 WorldViewProjection		: WorldViewProjection;	
float4x4 WorldViewInverse	: WorldViewInverse;		
float4x4 View				: View;	
float4x4 ViewProjection		: ViewProjection;	

#ifdef MOTION_BLUR_ENABLED
float4x4 PreviousWorld		: PreviousWorld;	
float4x4 PreviousWorldViewProjection : PreviousWorldViewProjection;
#endif

float cameraNear;
float cameraFar;
float cameraNearFar;
float cameraFarMinusNear;

#ifdef SKINNING_ENABLED
#define NUM_SKIN_TRANSFORMS 60
float4x4 BoneTransforms[NUM_SKIN_TRANSFORMS] : BONETRANSFORMS;
#endif

// Distance fog parameters.
#ifdef FOG_ENABLED
half4 FogColor : FOGCOLOR;
// FogRangeParameters : x = Near, y = Far, z = 1/(Far-Near)
float4 FogRangeParameters : FOGRANGEPARAMETERS;
#endif

// Tone mapping parameters.
#ifdef TONE_MAP_ENABLED
float SceneBrightnessScale;
float InvAverageVisibleLuminanceValueScaled;
#endif

float screenWidthInv : SCREENWIDTHINV;
float screenHeightInv : SCREENHEIGHTINV;
float3 EyePosition : EYEPOSITIONWS;

// Material Parameters
half4 MaterialColour : MaterialColour = half4(1.0f,1.0f,1.0f,1.0f);
// Material-supplied lighting Parameters.
half MaterialDiffuse : MaterialDiffuse = 1.0f;
half MaterialEmissiveness : MaterialEmissiveness = 0.0f;
#ifdef SPECULAR_ENABLED
half Shininess : Shininess 
<
float UIMin = 0; float UIMax = 20.0;
>  = 0.0f;
half SpecularPower : SpecularPower
<
float UIMin = 0; float UIMax = 64.0;
> = 16.0f;
half FresnelPower : FresnelPower
<
float UIMin = 0; float UIMax = 64.0;
>  = 5.0f;
#endif // SPECULAR_ENABLED

// Engine-supplied lighting parameters
#ifdef USE_LIGHTING
half3 GlobalAmbientColor : GlobalAmbientColor;

#if NUM_LIGHTS > 0
LIGHTTYPE_0 Light0 : LIGHT0;
#ifndef SHADOWTYPE_0
#define LightShadow0 0.0f
#else
SHADOWTYPE_0 LightShadow0 : LIGHTSHADOW0;
#endif
#endif
#if NUM_LIGHTS > 1
LIGHTTYPE_1 Light1 : LIGHT1;
#ifndef SHADOWTYPE_1
#define LightShadow1 0.0f
#else
SHADOWTYPE_1 LightShadow1 : LIGHTSHADOW1;
#endif
#endif
#if NUM_LIGHTS > 2
LIGHTTYPE_2 Light2 : LIGHT2;
#ifndef SHADOWTYPE_2
#define LightShadow2 0.0f
#else
SHADOWTYPE_2 LightShadow2 : LIGHTSHADOW2;
#endif
#endif
#if NUM_LIGHTS > 3
LIGHTTYPE_3 Light3 : LIGHT3;
#ifndef SHADOWTYPE_3
#define LightShadow3 0.0f
#else
SHADOWTYPE_3 LightShadow3 : LIGHTSHADOW3;
#endif
#endif

#endif

// Textures
#ifdef TEXTURE_ENABLED
sampler2D TextureSampler;
#endif // TEXTURE_ENABLED

#ifdef NORMAL_MAPPING_ENABLED
sampler2D NormalMapSampler;
#endif // NORMAL_MAPPING_ENABLED


#ifdef SSAO_ENABLED
sampler2D SSAOSampler;
#endif // SSAO_ENABLED

#ifdef LIGHTPREPASS_ENABLED
sampler2D LightPrepassSampler;
#endif

#ifdef LOD_BLEND
sampler2D DitherNoiseTexture;
half LodBlendValue : LodBlendValue;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Structures

#ifdef INSTANCING_ENABLED
struct InstancingInput
{
	float4 InstanceTransform0 : TEXCOORD5;
	float4 InstanceTransform1 : TEXCOORD6;
	float4 InstanceTransform2 : TEXCOORD7;
};
#endif

struct ZVSInput
{
#ifdef SKINNING_ENABLED
	float3 SkinnableVertex : POSITION;
#else
	float4 Position	: POSITION;
#endif
#ifdef SKINNING_ENABLED
	float4 SkinIndices : COLOR0;
	float4 SkinWeights : TEXCOORD2;
#endif
#ifdef INSTANCING_ENABLED
	InstancingInput instancingInput;
#endif
};


struct DefaultVSInput
{
#ifdef SKINNING_ENABLED
	float3 SkinnableVertex : POSITION;
	float3 SkinnableNormal	: NORMAL; 	
#else
	float4 Position	: POSITION;
	float3 Normal	: NORMAL; 	
#endif
	float2 Uv	: TEXCOORD0;
#ifdef USE_TANGENTS
#ifdef SKINNING_ENABLED
	float3 SkinnableTangent	: TEXCOORD1;
#else
	float3 Tangent	: TEXCOORD1;
#endif
#endif
#ifdef SKINNING_ENABLED
	float4 SkinIndices : COLOR0;
	float4 SkinWeights : TEXCOORD2;
#endif
#ifdef INSTANCING_ENABLED
	InstancingInput instancingInput;
#endif
};


struct DefaultVSForwardRenderOutput
{
	float4 Position	: POSITION;	
	float2 Uv	: TEXCOORD0;
	half3 Normal : TEXCOORD1; 	
	float4 WorldPositionDepth : TEXCOORD2;
#ifdef USE_TANGENTS
	half3 Tangent	: TEXCOORD3;
#endif
#ifdef __psp2__
	half4 Color : TEXCOORD4;
#else
	float4 Color : COLOR0;
#endif
};

struct DefaultPSForwardRenderInput
{
	float2 Uv	: TEXCOORD0;
	half3 Normal : TEXCOORD1; 	
	float4 WorldPositionDepth : TEXCOORD2;
#ifdef USE_TANGENTS
	half3 Tangent	: TEXCOORD3;
#endif
#ifdef __psp2__
	half4 Color : TEXCOORD4;
#else
	float4 Color : COLOR0;
#endif
	float4 ScreenPosition : WPOS;
};


struct DefaultVSDeferredRenderOutput
{
	float4 Position	: POSITION;	
	float2 Uv	: TEXCOORD0;
	half3 Normal : TEXCOORD1; 	
#ifndef __psp2__
	float4 WorldPositionDepth : TEXCOORD2;
#endif
#ifdef USE_TANGENTS
	half3 Tangent	: TEXCOORD3;
#endif
#ifdef __psp2__
	half4 Color : TEXCOORD2;
#else
	float4 Color : COLOR0;
#endif
};

struct DefaultPSDeferredRenderInput
{
	float2 Uv	: TEXCOORD0;
	half3 Normal : TEXCOORD1; 	
#ifndef __psp2__
	float4 WorldPositionDepth : TEXCOORD2;
#endif
#ifdef USE_TANGENTS
	half3 Tangent	: TEXCOORD3;
#endif
#ifdef __psp2__
	half4 Color : TEXCOORD2;
#else
	float4 Color : COLOR0;
#endif
	float4 ScreenPosition : WPOS;
};

struct DefaultPSLightPrepassRenderInput
{
	float4 Position	: POSITION;	
	float2 Uv	: TEXCOORD0;
	half3 Normal : TEXCOORD1; 	
	float4 WorldPositionDepth : TEXCOORD1;
#ifdef USE_TANGENTS
	float3 Tangent	: TEXCOORD2;
#endif
	float3 ViewPosition : TEXCOORD3;

	float4 ScreenPosition : WPOS;
};


#ifdef INSTANCING_ENABLED
void ApplyInstanceTransformVertex(InstancingInput IN, inout float3 toTransform)
{
	float3 instanceTransformedPosition;
	instanceTransformedPosition.x = dot(IN.InstanceTransform0, float4(toTransform,1));
	instanceTransformedPosition.y = dot(IN.InstanceTransform1, float4(toTransform,1));
	instanceTransformedPosition.z = dot(IN.InstanceTransform2, float4(toTransform,1));
	toTransform = instanceTransformedPosition;
}

void ApplyInstanceTransformNormal(InstancingInput IN, inout float3 toTransform)
{
	float3 instanceTransformedNormal;
	instanceTransformedNormal.x = dot(IN.InstanceTransform0.xyz, toTransform);
	instanceTransformedNormal.y = dot(IN.InstanceTransform1.xyz, toTransform);
	instanceTransformedNormal.z = dot(IN.InstanceTransform2.xyz, toTransform);
	toTransform = instanceTransformedNormal;
}
void ApplyInstanceTransform(inout DefaultVSInput IN)
{
#ifdef SKINNING_ENABLED
	ApplyInstanceTransformVertex(IN.instancingInput, IN.SkinnableVertex.xyz);
	ApplyInstanceTransformNormal(IN.instancingInput, IN.SkinnableNormal.xyz);
	#ifdef USE_TANGENTS
		ApplyInstanceTransformNormal(IN.instancingInput, IN.SkinnableTangent.xyz);
	#endif
#else
	ApplyInstanceTransformVertex(IN.instancingInput, IN.Position.xyz);
	ApplyInstanceTransformNormal(IN.instancingInput, IN.Normal.xyz);
	#ifdef USE_TANGENTS
		ApplyInstanceTransformNormal(IN.instancingInput, IN.Tangent.xyz);
	#endif
#endif
}
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Vertex shaders

// Default shadow vertex shader.
float4 DefaultShadowVS(ZVSInput IN) : POSITION
{
#ifdef SKINNING_ENABLED
	float3 position = IN.SkinnableVertex;
	IN.SkinIndices *= 255.00001;
#ifdef __psp2__
	EvaluateSkinPosition2Bones(position.xyz, IN.SkinWeights, IN.SkinIndices, BoneTransforms );	
#else // __psp2__
	EvaluateSkinPosition4Bones(position.xyz, IN.SkinWeights, IN.SkinIndices, BoneTransforms );	
#endif // __psp2__
#else 
	float3 position = IN.Position.xyz;
#endif
#ifdef INSTANCING_ENABLED
	ApplyInstanceTransformVertex(IN.instancingInput, position);
#endif

#ifdef SKINNING_ENABLED
	return mul(ViewProjection, float4(position.xyz,1));
#else
	return mul(WorldViewProjection, float4(position.xyz,1));
#endif
}

// Default Z prepass vertex shader.
float4 DefaultZPrePassVS(ZVSInput IN) : POSITION
{
#ifdef SKINNING_ENABLED
	float3 position = IN.SkinnableVertex;
	IN.SkinIndices *= 255.00001;
	EvaluateSkinPosition4Bones(position.xyz, IN.SkinWeights, IN.SkinIndices, BoneTransforms );
#else 
	float3 position = IN.Position.xyz;
#endif

#ifdef INSTANCING_ENABLED
	ApplyInstanceTransformVertex(IN.instancingInput, position);
#endif

#ifdef SKINNING_ENABLED
	return mul(ViewProjection, float4(position.xyz,1));
#else
	return mul(WorldViewProjection, float4(position.xyz,1));
#endif
}

// Default forward render vertex shader
DefaultVSForwardRenderOutput DefaultForwardRenderVS(DefaultVSInput IN)
{
	DefaultVSForwardRenderOutput OUT;

#ifdef SKINNING_ENABLED
	IN.SkinIndices *= 255.00001;
	#ifdef USE_TANGENTS
		EvaluateSkinPositionNormalTangent4Bones(IN.SkinnableVertex.xyz, IN.SkinnableNormal.xyz, IN.SkinnableTangent.xyz, IN.SkinWeights, IN.SkinIndices, BoneTransforms );
	#else
		EvaluateSkinPositionNormal4Bones(IN.SkinnableVertex.xyz, IN.SkinnableNormal.xyz, IN.SkinWeights, IN.SkinIndices, BoneTransforms );
	#endif
	#ifdef INSTANCING_ENABLED
		ApplyInstanceTransform(IN);
	#endif
	float3 position = IN.SkinnableVertex.xyz;
	OUT.Position = mul(ViewProjection, float4(position,1.0f));
	OUT.WorldPositionDepth = float4(position.xyz, -mul(View, float4(position,1.0f)).z);
	OUT.Normal = IN.SkinnableNormal.xyz;
	#ifdef USE_TANGENTS
		OUT.Tangent = IN.SkinnableTangent.xyz;
	#endif
#else
	#ifdef INSTANCING_ENABLED
		ApplyInstanceTransform(IN);
	#endif
		float3 position = IN.Position.xyz;
		OUT.Position = mul(WorldViewProjection, float4(position,1.0f));
		OUT.WorldPositionDepth = float4( mul(World, float4(position,1.0f)).xyz, -mul(WorldView, float4(position,1.0f)).z);
		OUT.Normal = mul(World, float4(IN.Normal,0)).xyz;
	#ifdef USE_TANGENTS
		OUT.Tangent = mul(World, float4(IN.Tangent,0)).xyz;
	#endif
#endif		

	OUT.Uv.xy = IN.Uv;
	OUT.Color = 1.0f;
    
	return OUT;
}

// Default forward render vertex shader
DefaultVSDeferredRenderOutput DefaultDeferredRenderVS(DefaultVSInput IN)
{
	DefaultVSDeferredRenderOutput OUT;

#ifdef SKINNING_ENABLED
	IN.SkinIndices *= 255.00001;
	#ifdef USE_TANGENTS
		EvaluateSkinPositionNormalTangent4Bones(IN.SkinnableVertex.xyz, IN.SkinnableNormal.xyz, IN.SkinnableTangent.xyz, IN.SkinWeights, IN.SkinIndices, BoneTransforms );
	#else
		EvaluateSkinPositionNormal4Bones(IN.SkinnableVertex.xyz, IN.SkinnableNormal.xyz, IN.SkinWeights, IN.SkinIndices, BoneTransforms );
	#endif
	#ifdef INSTANCING_ENABLED
		ApplyInstanceTransform(IN);
	#endif
	float3 position = IN.SkinnableVertex.xyz;
	OUT.Position = mul(ViewProjection, float4(position,1.0f));
#ifndef __psp2__
	OUT.WorldPositionDepth = float4( position.xyz, -mul(View, float4(position,1.0f)).z);
#endif
	OUT.Normal = normalize(mul(View, float4(IN.SkinnableNormal.xyz,0)).xyz);
	#ifdef USE_TANGENTS
		OUT.Tangent = normalize(mul(View, float4(IN.SkinnableTangent.xyz,0)).xyz);
	#endif
#else
	#ifdef INSTANCING_ENABLED
		ApplyInstanceTransform(IN);
	#endif
	float3 position = IN.Position.xyz;
	OUT.Position = mul(WorldViewProjection, float4(position,1.0));
#ifndef __psp2__
	OUT.WorldPositionDepth = float4( mul(World, float4(position,1.0)).xyz, -mul(WorldView, float4(position,1.0)).z);
#endif
	OUT.Normal = normalize(mul(WorldView, float4(IN.Normal,0)).xyz);
	#ifdef USE_TANGENTS
		OUT.Tangent = normalize(mul(WorldView, float4(IN.Tangent,0)).xyz);
	#endif
#endif		

	OUT.Uv.xy = IN.Uv;
	OUT.Color = MaterialColour * float4(MaterialDiffuse,MaterialDiffuse,MaterialDiffuse,1.0);
    
	return OUT;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Fragment utility macros. Just make the fragment shader code a bit neater by hiding some of the combination handling / argument passing here.

#ifdef PARALLAX_OFFSET_MAPPING_ENABLED
#define EvaluateNormal(In) EvaluateParallaxMapNormal(In.Normal.xyz,In.Uv,In.Tangent, NormalMapSampler, normalize(mul(View, float4(IN.WorldPositionDepth.xyz,1)).xyz))
#elif defined(NORMAL_MAPPING_ENABLED)
#define EvaluateNormal(In) EvaluateNormalMapNormal(In.Normal.xyz,In.Uv,In.Tangent, NormalMapSampler)
#else
#define EvaluateNormal(In) EvaluateStandardNormal(In.Normal.xyz)
#endif

#ifdef RECEIVE_SHADOWS
#define EvaluateShadowValue(LightId, LightShadowId, worldPos, viewDepth) EvaluateShadow(LightId, LightShadowId, worldPos, viewDepth)
#else
#define EvaluateShadowValue(LightId, LightShadowId, worldPos, viewDepth) 1.0h
#endif

#ifdef SPECULAR_ENABLED
#define EvaluateLightFunction(LightIndex) \
	{ \
		half shad = EvaluateShadowValue(Light##LightIndex, LightShadow##LightIndex, worldPosition, In.WorldPositionDepth.w); \
		lightResult += EvaluateLight(Light##LightIndex, worldPosition, normal, -eyeDirection, shad, shininess, SpecularPower/*, FresnelPower*/); \
	}
#else //! SPECULAR_ENABLED
#define EvaluateLightFunction(LightIndex) \
	{ \
		half shad = EvaluateShadowValue(Light##LightIndex, LightShadow##LightIndex, worldPosition, In.WorldPositionDepth.w); \
		lightResult += EvaluateLight(Light##LightIndex, worldPosition, normal, shad); \
	}
#endif //! SPECULAR_ENABLED

#ifdef LOD_BLEND
half GetLODDitherValue(float2 screenUv)
{
	//return screenUv.x > (1.0f-LodBlendValue) ? 1.0f : -1.0f;

	half4 ditherValue = tex2D(DitherNoiseTexture, screenUv);
	half threshold = (1.0h-abs(LodBlendValue));

	half rslt = LodBlendValue >= 0 ? ((ditherValue.x >= threshold) ? 1.0h : -1.0h) : ((ditherValue.x < threshold) ? 1.0h : -1.0h);
	return rslt;
	//return ((ditherValue.x >= threshold) ? 1.0f : -1.0f) * sign(LodBlendValue);
	//return (ditherValue.x > (1.0f-abs(LodBlendValue)) ? 1.0f : -1.0f) * sign(LodBlendValue);
}
#endif

// Tone mapping.
#ifdef TONE_MAP_ENABLED

half4 ToneMap(half3 colourValue)
{
	half lum = dot(colourValue,half3(0.299h,0.587h,0.144h)) ;
	half lumToneMap = lum * InvAverageVisibleLuminanceValueScaled;
    half lumToneMap1 = lumToneMap + 1;
    half lD = (lumToneMap*(1 + (lumToneMap * SceneBrightnessScale)))/lumToneMap1;
    
	// divide by luminance 
    half3 colourResult = colour / lum;
   	colourResult *= lD;
   
	return float4(colourResult, (lum * SceneBrightnessScale) + (1.0f/255.0f));
}

#endif

// Fog.
#ifdef FOG_ENABLED

half3 EvaluateFog(half3 colourValue, float3 viewPosition)
{
	half fogAmt = saturate((abs(viewPosition.z)-FogRangeParameters.x) * FogRangeParameters.z);
	return lerp(colourValue,FogColor.xyz,fogAmt);
}

#endif


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Fragment shaders

// Default fragment shader. 
float4 DefaultUnshadedFP(float4 ScreenPosition : WPOS) : COLOR0
{
#ifdef LOD_BLEND
	clip(GetLODDitherValue(ScreenPosition.xy * (1.0f/256.0f)));
#endif
	return 1;
}

// Default fragment shader. 
#ifdef __psp2__
//__nativecolor __regformat unsigned char4 DefaultShadowFP()
half4 DefaultShadowFP() : COLOR0
#else
float4 DefaultShadowFP() : COLOR0
#endif
{
#ifdef __psp2__
	return unsigned char4(0,0,0,0);
#else
	return 0.0;
#endif
}



half4 PackNormalAndViewSpaceDepth(half3 normal, float viewSpaceZ)
{
	float normalizedViewZ = viewSpaceZ / cameraFar;
	half2 depthPacked = half2( floor(normalizedViewZ * 256.0f) / 255.0f,  frac(normalizedViewZ * 256.0f) );
	half4 rslt = half4(normal.xy, depthPacked);
	return rslt;
}
half4 PackNormalAndDepth(half3 normal, float depth)
{
	float viewSpaceZ = -(cameraNearFar / (depth * cameraFarMinusNear - cameraFar));
	return PackNormalAndViewSpaceDepth(normal,viewSpaceZ);
}

// Default light pre pass first pass shader. Outputs normal and defaulted specular power only.
float4 DefaultLightPrepassFP(DefaultVSForwardRenderOutput In) : COLOR0
{
	half3 normal = EvaluateNormal(In);
#ifdef SPECULAR_ENABLED
	half specPower = SpecularPower;
#else
	half specPower = 0.0f;
#endif

	return half4(normal.xyz * 0.5f+0.5f,specPower);
}


// Deferred rendering
#ifdef __psp2__
__nativecolor __regformat half4 DefaultDeferredRenderFP(DefaultVSDeferredRenderOutput In)
#else // __psp2__
PSDeferredOutput DefaultDeferredRenderFP(DefaultVSDeferredRenderOutput In, float4 ScreenPosition : WPOS) 
#endif // __psp2__
{
	half3 normal = EvaluateNormal(In);

#ifdef SPECULAR_ENABLED
	// could vary by pixel with a texture lookup
	half specPower = SpecularPower;
	half gloss = 1;
#else // SPECULAR_ENABLED
	half specPower = 0;
	half gloss = 0;
#endif // SPECULAR_ENABLED

	half4 colour = In.Color;
#ifdef TEXTURE_ENABLED
#ifdef __psp2__
	colour *= tex2D<half4>(TextureSampler, In.Uv);
#else // __psp2__
	colour *= h4tex2D(TextureSampler, In.Uv);
#endif // __psp2__
#endif // TEXTURE_ENABLED

	half3 viewSpaceNormal = normal;
				
#ifdef __psp2__
	
	unsigned char4 out0 = unsigned char4(colour * 255.0);

	half4 viewNormal = half4(viewSpaceNormal.xyzz * 0.5 + 0.5);
	unsigned char4 out1 = unsigned char4(viewNormal* 255.0);

	unsigned int2 outputValue =  unsigned int2(bit_cast<unsigned int>(out0), bit_cast<unsigned int>(out1));		
	return bit_cast<half4>(outputValue);
		
#else // __psp2__

#ifdef LOD_BLEND
	clip(GetLODDitherValue(ScreenPosition.xy * (1.0f/256.0f)));
#endif // LOD_BLEND
	PSDeferredOutput Out;
	Out.Colour = colour;
	Out.NormalDepth = PackNormalAndViewSpaceDepth(half3(viewSpaceNormal.xy*0.5f+0.5f,0), In.WorldPositionDepth.w);
	return Out;	
#endif // __psp2__

}


float3 EvaluateLightingDefault( DefaultPSForwardRenderInput In, float3 worldPosition, half3 normal, half glossValue )
{
	// Lighting
	half3 lightResult = 1;

#ifdef USE_LIGHTING
	lightResult = GlobalAmbientColor;

#ifdef SPECULAR_ENABLED
	half3 eyeDirection = normalize(worldPosition - EyePosition);
	half shininess = Shininess * glossValue;
#endif
	
#if NUM_LIGHTS > 0
	EvaluateLightFunction(0);
#endif
#if NUM_LIGHTS > 1
	EvaluateLightFunction(1);
#endif
#if NUM_LIGHTS > 2
	EvaluateLightFunction(2);
#endif
#if NUM_LIGHTS > 3
	EvaluateLightFunction(3);
#endif
#endif

	return lightResult;
}


#endif

