/* SCE CONFIDENTIAL
PhyreEngine(TM) Package 3.1.0.0
* Copyright (C) 2011 Sony Computer Entertainment Inc.
* All Rights Reserved.
*/

///////////////////////////////////////////////////////////////////////////////
// Defines to enable features.
//
// TEX - Number of textures to sample (0, 1 or 2).
// VCOLOR - Use vertex color.
// CXFORM - Use scale and offset color transform.
// FACTORA - Apply Factor.a modulation.
// CMUL - Apply color multiply (lerp from white).
//
// VPNAME - Name of vertex program.
// FPNAME - Name of fragment program.
// TECHNIQUENAME - Name of technique.

///////////////////////////////////////////////////////////////////////////////
// Shader code.

float4 FPNAME(VPOutput IN) : COLOR0
{
	float4 color;

#if (TEX > 0)
	#if defined(TEX_YUV)
		float4 c = float4((tex2D(tex_y, IN.Tex0).r - 16.f/255.f) * 1.164f);	// Grab YUV from texture planes and color space convert.
		float4 U = float4(tex2D(tex_u, IN.Tex0).r - 128.f/255.f);
		float4 V = float4(tex2D(tex_v, IN.Tex0).r - 128.f/255.f);
		c += V * float4(1.596f, -0.813f, 0, 0);
		c += U * float4(0, -0.392f, 2.017f, 0);
		c.a = 1.0f;
		color = c;
		#if defined (TEX_YUVA)
			color.a = tex2D(tex_a, IN.Tex0).r;								// Assign A from alpha plane.
		#endif //! YEX_YUVA
	#else //! TEX_YUV
		float4 colorTex0 = tex2D(tex0, IN.Tex0);							// Sample texture 0
		color = colorTex0;
	#endif //! TEX_YUV
	#if (TEX > 1)
		float4 colorTex1 = tex2D(tex1, IN.Tex1);							// Sample and blend texture 1
		color = lerp(colorTex1, color, IN.Factor.r);
	#endif //! (TEX > 1)
	#if defined VCOLOR
		color = lerp(IN.Color, color, IN.Factor.r);							// Blend in vertex color
	#endif //! VCOLOR
#elif defined(VCOLOR)
	color = IN.Color;														// Grab vertex color.
#else //! VCOLOR
	color = float4(1);														// No idea - use white.
#endif // !Color

#if defined(CXFORM)
	color = color * IN.Fucxmul + IN.Fucxadd;								// Apply color transform.
#endif //! CXFORM

#ifdef FACTORA
	color.a *= IN.Factor.g;													// Apply factor a
#endif //! FACTORA

#ifdef CMUL
	color = lerp(float4(1), color, color.a);								// Apply color multiply.
#endif //! CMUL
  
  return color;
}

///////////////////////////////////////////////////////////////////////////////
// Techniques

technique TECHNIQUENAME
{
	pass main
	{
		VertexProgram = compile vp40 VPNAME();
		FragmentProgram = compile fp40 FPNAME();	
	}
}

#undef TEX
#undef TEX_YUV
#undef TEX_YUVA
#undef VCOLOR
#undef CXFORM
#undef FACTORA
#undef CMUL

#undef VPNAME
#undef FPNAME
#undef TECHNIQUENAME
