/* SCE CONFIDENTIAL
PhyreEngine(TM) Package 3.1.0.0
* Copyright (C) 2011 Sony Computer Entertainment Inc.
* All Rights Reserved.
*/

///////////////////////////////////////////////////////////////////////////////
// Defines to enable features.
//
// POSITION_D=2 - Enable 2D position (screen space).
// POSITION_D=3 - Enable 3D position (world space).
// TEXGEN=1 - Enable 1 set of texture coordinate generation.
// TEXGEN=2 - Enable 2 sets of texture coordinate generation.
// TC=1 - Enable 1 set of specified texture coordinates.
// TC=2 - Enable 2 sets of specified texture coordinates.
// CXADD - Enable batched color transform add.
// CXMUL - Enable batched color transform multiply.
// COLOR - Enable color passthrough.
// FACTOR - Enable factor passthrough.
// VPNAME - Name of vertex program.

///////////////////////////////////////////////////////////////////////////////
// Shader code.

// Vertex uber-shader for Scaleform
VPOutput VPNAME (VPInput IN)
{
	VPOutput OUT;

	// Calculate stride of uniforms and assign offsets
	int batchStride = 0;
#if (TEXGEN > 0)
	int texgenOffset = batchStride;
	batchStride = batchStride + 2;
#endif // TEXGEN > 0
#if (TEXGEN > 1)
	batchStride = batchStride + 2;
#endif // TEXGEN > 1

#ifdef CXADD
	int cxaddOffset = batchStride;
	batchStride = batchStride + 1;
#endif //! CXADD
#ifdef CXMUL
	int cxmulOffset = batchStride;
	batchStride = batchStride + 1;
#endif //! CXMUL

#if (POSITION_D == 2)
	int mvpOffset = batchStride;
	batchStride = batchStride + 2;
#endif // POSITION_D

	int batch = (int)(IN.FactorBatch.z * 255.00001f);
	int batchOffset = batchStride * batch;

#if (POSITION_D == 2)
	float4 outPos = IN.Position;
	outPos.x = dot(IN.Position, vfuniforms[batchOffset + mvpOffset + 0]);
	outPos.y = dot(IN.Position, vfuniforms[batchOffset + mvpOffset + 1]);
	OUT.Position = outPos;
#elif (POSITION_D == 3)
	OUT.Position = mul(vfmuniforms[batch], IN.Position);
#else //! POSITION_D
	OUT.Position = float4(0);
#endif //! POSITION_D

#if (TC > 0)
	OUT.Tex0 = IN.Tex0;
#elif (TEXGEN > 0)
	OUT.Tex0.x = dot(IN.Position, vfuniforms[batchOffset + texgenOffset + 0]);
	OUT.Tex0.y = dot(IN.Position, vfuniforms[batchOffset + texgenOffset + 1]);
#else // TC/TEXGEN
	OUT.Tex0 = float2(0);
#endif //! TC/TEXGEN

#if (TEXGEN > 1)
	OUT.Tex1.x = dot(IN.Position, vfuniforms[batchOffset + texgenOffset + 2]);
	OUT.Tex1.y = dot(IN.Position, vfuniforms[batchOffset + texgenOffset + 3]);
#else //! TEXGEN
	OUT.Tex1 = float2(0);
#endif //! TEXGEN

#ifdef CXADD
	OUT.Fucxadd = vfuniforms[batchOffset + cxaddOffset];
#else //! CXADD
	OUT.Fucxadd = float4(0);
#endif //! CXADD

#ifdef CXMUL
	OUT.Fucxmul = vfuniforms[batchOffset + cxmulOffset];
#else //! CXMUL
	OUT.Fucxmul = float4(0);
#endif //! CXMUL

#ifdef COLOR
	OUT.Color = IN.Color;
#else //! COLOR
	OUT.Color = half4(0);
#endif //! COLOR

#ifdef FACTOR
	OUT.Factor = IN.FactorBatch.xy;
#else //! FACTOR
	OUT.Factor = half2(0);
#endif //! FACTOR

	return OUT;
}

#undef TEXGEN
#undef TC
#undef CXADD
#undef CXMUL
#undef COLOR
#undef FACTOR
#undef VPNAME
