/* SCE CONFIDENTIAL
PhyreEngine(TM) Package 3.1.0.0
* Copyright (C) 2011 Sony Computer Entertainment Inc.
* All Rights Reserved.
*/

///////////////////////////////////////////////////////////////////////////////
// Defines to enable features.
//
// POSITION_D=2 - Enable 2D position (screen space).
// POSITION_D=3 - Enable 3D position (world space).
// TEXGEN=1 - Enable 1 set of texture coordinate generation.
// TEXGEN=2 - Enable 2 sets of texture coordinate generation.
// TC=1 - Enable 1 set of specified texture coordinates.
// TC=2 - Enable 2 sets of specified texture coordinates.
// COLOR - Enable color passthrough.
// FACTOR - Enable factor passthrough.
// VPNAME - Name of vertex program.

///////////////////////////////////////////////////////////////////////////////
// Shader code.

// Vertex uber-shader for Scaleform
VPOutput VPNAME (VPInput IN)
{
	VPOutput OUT;

#if (POSITION_D == 2)
	float4 outPos = IN.Position;
	outPos.x = dot(IN.Position, mvp[0]);
	outPos.y = dot(IN.Position, mvp[1]);
	OUT.Position = outPos;
#elif (POSITION_D == 3)
	OUT.Position = mul(mvp, IN.Position);
#else //! POSITION_D
	OUT.Position = float4(0);
#endif //! POSITION_D

#if (TC > 0)
	OUT.Tex0 = IN.Tex0;
#elif (TEXGEN > 0)
	OUT.Tex0.x = dot(IN.Position, texgen[0]);
	OUT.Tex0.y = dot(IN.Position, texgen[1]);
#else //! TC/TEXGEN
	OUT.Tex0 = float2(0);
#endif //! TC/TEXGEN

#if (TEXGEN > 1)
	OUT.Tex1.x = dot(IN.Position, texgen[2]);
	OUT.Tex1.y = dot(IN.Position, texgen[3]);
#else //! (TEXGEN > 1)
	OUT.Tex1 = float2(0);
#endif //! (TEXGEN > 1)

#ifdef COLOR
	OUT.Color = IN.Color;
#else //! COLOR
	OUT.Color = half4(0);
#endif //! COLOR

#ifdef FACTOR
	OUT.Factor = IN.FactorBatch.xy;
#else //! FACTOR
	OUT.Factor = half2(0);
#endif //! FACTOR

	return OUT;
}

#undef TEXGEN
#undef TC
#undef COLOR
#undef FACTOR
#undef VPNAME
